@@ -1,12 +1,15 @@
 use super::{RemoteProcess, RemoteProcessError, RemoteStderr, RemoteStdin, RemoteStdout};
-use crate::client::Session;
+use crate::client::SessionChannel;
 use futures::stream::{Stream, StreamExt};
 use std::{
     fmt::Write,
     io::{self, Cursor, Read},
     ops::{Deref, DerefMut},
 };
-use tokio::{sync::mpsc, task::JoinHandle};
+use tokio::{
+    sync::mpsc::{self, error::TryRecvError},
+    task::JoinHandle,
+};
 
 mod data;
 pub use data::*;
@@ -25,11 +28,12 @@ impl RemoteLspProcess {
     /// the process like an LSP server
     pub async fn spawn(
         tenant: impl Into<String>,
-        session: &mut Session,
+        channel: SessionChannel,
         cmd: impl Into<String>,
         args: Vec<String>,
+        detached: bool,
     ) -> Result<Self, RemoteProcessError> {
-        let mut inner = RemoteProcess::spawn(tenant, session, cmd, args).await?;
+        let mut inner = RemoteProcess::spawn(tenant, channel, cmd, args, detached).await?;
         let stdin = inner.stdin.take().map(RemoteLspStdin::new);
         let stdout = inner.stdout.take().map(RemoteLspStdout::new);
         let stderr = inner.stderr.take().map(RemoteLspStderr::new);
@@ -74,18 +78,24 @@ impl RemoteLspStdin {
         Self { inner, buf: None }
     }
 
-    /// Writes data to the stdin of a specific remote process
-    pub async fn write(&mut self, data: &str) -> io::Result<()> {
-        // Create or insert into our buffer
-        match &mut self.buf {
-            Some(buf) => buf.push_str(data),
-            None => self.buf = Some(data.to_string()),
+    /// Tries to write data to the stdin of a specific remote process
+    pub fn try_write(&mut self, data: &str) -> io::Result<()> {
+        let queue = self.update_and_read_messages(data)?;
+
+        // Process and then send out each LSP message in our queue
+        for mut data in queue {
+            // Convert distant:// to file://
+            data.mut_content().convert_distant_scheme_to_local();
+            data.refresh_content_length();
+            self.inner.try_write(&data.to_string())?;
         }
 
-        // Read LSP messages from our internal buffer
-        let buf = self.buf.take().unwrap();
-        let (remainder, queue) = read_lsp_messages(buf)?;
-        self.buf = remainder;
+        Ok(())
+    }
+
+    /// Writes data to the stdin of a specific remote process
+    pub async fn write(&mut self, data: &str) -> io::Result<()> {
+        let queue = self.update_and_read_messages(data)?;
 
         // Process and then send out each LSP message in our queue
         for mut data in queue {
@@ -97,6 +107,30 @@ impl RemoteLspStdin {
 
         Ok(())
     }
+
+    fn update_and_read_messages(&mut self, data: &str) -> io::Result<Vec<LspData>> {
+        // Create or insert into our buffer
+        match &mut self.buf {
+            Some(buf) => buf.push_str(data),
+            None => self.buf = Some(data.to_string()),
+        }
+
+        // Read LSP messages from our internal buffer
+        let buf = self.buf.take().unwrap();
+        match read_lsp_messages(&buf) {
+            // If we succeed, update buf with our remainder and return messages
+            Ok((remainder, queue)) => {
+                self.buf = remainder;
+                Ok(queue)
+            }
+
+            // Otherwise, if failed, reset buf back to what it was
+            Err(x) => {
+                self.buf = Some(buf);
+                Err(x)
+            }
+        }
+    }
 }
 
 /// A handle to a remote LSP process' standard output (stdout)
@@ -121,6 +155,18 @@ impl RemoteLspStdout {
         Self { read_task, rx }
     }
 
+    /// Tries to read a complete LSP message over stdout, returning `None` if no complete message
+    /// is available
+    pub fn try_read(&mut self) -> io::Result<Option<String>> {
+        match self.rx.try_recv() {
+            Ok(Ok(data)) => Ok(Some(data)),
+            Ok(Err(x)) => Err(x),
+            Err(TryRecvError::Empty) => Ok(None),
+            Err(TryRecvError::Disconnected) => Err(io::Error::from(io::ErrorKind::BrokenPipe)),
+        }
+    }
+
+    /// Reads a complete LSP message over stdout
     pub async fn read(&mut self) -> io::Result<String> {
         self.rx
             .recv()
@@ -158,6 +204,18 @@ impl RemoteLspStderr {
         Self { read_task, rx }
     }
 
+    /// Tries to read a complete LSP message over stderr, returning `None` if no complete message
+    /// is available
+    pub fn try_read(&mut self) -> io::Result<Option<String>> {
+        match self.rx.try_recv() {
+            Ok(Ok(data)) => Ok(Some(data)),
+            Ok(Err(x)) => Err(x),
+            Err(TryRecvError::Empty) => Ok(None),
+            Err(TryRecvError::Disconnected) => Err(io::Error::from(io::ErrorKind::BrokenPipe)),
+        }
+    }
+
+    /// Reads a complete LSP message over stderr
     pub async fn read(&mut self) -> io::Result<String> {
         self.rx
             .recv()
@@ -190,7 +248,7 @@ where
 
             // Read LSP messages from our internal buffer
             let buf = task_buf.take().unwrap();
-            let (remainder, queue) = match read_lsp_messages(buf) {
+            let (remainder, queue) = match read_lsp_messages(&buf) {
                 Ok(x) => x,
                 Err(x) => {
                     let _ = tx.send(Err(x)).await;
@@ -218,7 +276,7 @@ where
     (read_task, rx)
 }
 
-fn read_lsp_messages(input: String) -> io::Result<(Option<String>, Vec<LspData>)> {
+fn read_lsp_messages(input: &str) -> io::Result<(Option<String>, Vec<LspData>)> {
     let mut queue = Vec::new();
 
     // Continue to read complete messages from the input until we either fail to parse or we reach
@@ -248,6 +306,7 @@ fn read_lsp_messages(input: String) -> io::Result<(Option<String>, Vec<LspData>)
 mod tests {
     use super::*;
     use crate::{
+        client::Session,
         data::{Request, RequestData, Response, ResponseData},
         net::{InmemoryStream, PlainCodec, Transport},
     };
@@ -259,13 +318,14 @@ mod tests {
     // Configures an lsp process with a means to send & receive data from outside
     async fn spawn_lsp_process() -> (Transport<InmemoryStream, PlainCodec>, RemoteLspProcess) {
         let (mut t1, t2) = Transport::make_pair();
-        let mut session = Session::initialize(t2).unwrap();
+        let session = Session::initialize(t2).unwrap();
         let spawn_task = tokio::spawn(async move {
             RemoteLspProcess::spawn(
                 String::from("test-tenant"),
-                &mut session,
+                session.clone_channel(),
                 String::from("cmd"),
                 vec![String::from("arg")],
+                false,
             )
             .await
         });
