@@ -1,22 +1,28 @@
 use crate::{
-    client::Session,
-    constants::CLIENT_BROADCAST_CHANNEL_CAPACITY,
-    data::{Request, RequestData, Response, ResponseData},
-    net::{DataStream, TransportError},
+    client::{Mailbox, SessionChannel},
+    constants::CLIENT_PIPE_CAPACITY,
+    data::{Request, RequestData, ResponseData},
+    net::TransportError,
 };
 use derive_more::{Display, Error, From};
 use log::*;
+use std::sync::Arc;
 use tokio::{
     io,
-    sync::mpsc,
+    sync::{
+        mpsc::{
+            self,
+            error::{TryRecvError, TrySendError},
+        },
+        RwLock,
+    },
     task::{JoinError, JoinHandle},
 };
 
+type StatusResult = Result<(bool, Option<i32>), RemoteProcessError>;
+
 #[derive(Debug, Display, Error, From)]
 pub enum RemoteProcessError {
-    /// When the process receives an unexpected response
-    BadResponse,
-
     /// When attempting to relay stdout/stderr over channels, but the channels fail
     ChannelDead,
 
@@ -37,12 +43,14 @@ pub struct RemoteProcess {
     /// Id of the process
     id: usize,
 
-    /// Task that forwards stdin to the remote process by bundling it as stdin requests
-    req_task: JoinHandle<Result<(), RemoteProcessError>>,
+    /// Id used to map back to mailbox
+    pub(crate) origin_id: usize,
 
-    /// Task that reads in new responses, which returns the success and optional
-    /// exit code once the process has completed
-    res_task: JoinHandle<Result<(bool, Option<i32>), RemoteProcessError>>,
+    // Sender to abort req task
+    abort_req_task_tx: mpsc::Sender<()>,
+
+    // Sender to abort res task
+    abort_res_task_tx: mpsc::Sender<()>,
 
     /// Sender for stdin
     pub stdin: Option<RemoteStdin>,
@@ -55,68 +63,129 @@ pub struct RemoteProcess {
 
     /// Sender for kill events
     kill: mpsc::Sender<()>,
+
+    /// Task that waits for the process to complete
+    wait_task: JoinHandle<()>,
+
+    /// Handles the success and exit code for a completed process
+    status: Arc<RwLock<Option<StatusResult>>>,
 }
 
 impl RemoteProcess {
     /// Spawns the specified process on the remote machine using the given session
-    pub async fn spawn<T>(
-        tenant: String,
-        mut session: Session<T>,
-        cmd: String,
+    pub async fn spawn(
+        tenant: impl Into<String>,
+        mut channel: SessionChannel,
+        cmd: impl Into<String>,
         args: Vec<String>,
-    ) -> Result<Self, RemoteProcessError>
-    where
-        T: DataStream + 'static,
-    {
-        // Submit our run request and wait for a response
-        let res = session
-            .send(Request::new(
+        detached: bool,
+    ) -> Result<Self, RemoteProcessError> {
+        let tenant = tenant.into();
+        let cmd = cmd.into();
+
+        // Submit our run request and get back a mailbox for responses
+        let mut mailbox = channel
+            .mail(Request::new(
                 tenant.as_str(),
-                vec![RequestData::ProcRun { cmd, args }],
+                vec![RequestData::ProcRun {
+                    cmd,
+                    args,
+                    detached,
+                }],
             ))
             .await?;
 
-        // We expect a singular response back
-        if res.payload.len() != 1 {
-            return Err(RemoteProcessError::BadResponse);
-        }
-
-        // Response should be proc starting
-        let id = match res.payload.into_iter().next().unwrap() {
-            ResponseData::ProcStart { id } => id,
-            _ => return Err(RemoteProcessError::BadResponse),
+        // Wait until we get the first response, and get id from proc started
+        let (id, origin_id) = match mailbox.next().await {
+            Some(res) if res.payload.len() != 1 => {
+                return Err(RemoteProcessError::TransportError(TransportError::IoError(
+                    io::Error::new(io::ErrorKind::InvalidData, "Got wrong payload size"),
+                )));
+            }
+            Some(res) => {
+                let origin_id = res.origin_id;
+                match res.payload.into_iter().next().unwrap() {
+                    ResponseData::ProcStart { id } => (id, origin_id),
+                    ResponseData::Error(x) => {
+                        return Err(RemoteProcessError::TransportError(TransportError::IoError(
+                            x.into(),
+                        )))
+                    }
+                    x => {
+                        return Err(RemoteProcessError::TransportError(TransportError::IoError(
+                            io::Error::new(
+                                io::ErrorKind::InvalidData,
+                                format!("Got response type of {}", x.as_ref()),
+                            ),
+                        )))
+                    }
+                }
+            }
+            None => {
+                return Err(RemoteProcessError::TransportError(TransportError::IoError(
+                    io::Error::from(io::ErrorKind::ConnectionAborted),
+                )))
+            }
         };
 
         // Create channels for our stdin/stdout/stderr
-        let (stdin_tx, stdin_rx) = mpsc::channel(CLIENT_BROADCAST_CHANNEL_CAPACITY);
-        let (stdout_tx, stdout_rx) = mpsc::channel(CLIENT_BROADCAST_CHANNEL_CAPACITY);
-        let (stderr_tx, stderr_rx) = mpsc::channel(CLIENT_BROADCAST_CHANNEL_CAPACITY);
+        let (stdin_tx, stdin_rx) = mpsc::channel(CLIENT_PIPE_CAPACITY);
+        let (stdout_tx, stdout_rx) = mpsc::channel(CLIENT_PIPE_CAPACITY);
+        let (stderr_tx, stderr_rx) = mpsc::channel(CLIENT_PIPE_CAPACITY);
 
         // Used to terminate request task, either explicitly by the process or internally
         // by the response task when it terminates
         let (kill_tx, kill_rx) = mpsc::channel(1);
+        let kill_tx_2 = kill_tx.clone();
 
         // Now we spawn a task to handle future responses that are async
         // such as ProcStdout, ProcStderr, and ProcDone
-        let kill_tx_2 = kill_tx.clone();
-        let broadcast = session.broadcast.take().unwrap();
+        let (abort_res_task_tx, mut abort_res_task_rx) = mpsc::channel::<()>(1);
         let res_task = tokio::spawn(async move {
-            process_incoming_responses(id, broadcast, stdout_tx, stderr_tx, kill_tx_2).await
+            tokio::select! {
+                _ = abort_res_task_rx.recv() => {
+                    panic!("killed");
+                }
+                res = process_incoming_responses(id, mailbox, stdout_tx, stderr_tx, kill_tx_2) => {
+                    res
+                }
+            }
         });
 
         // Spawn a task that takes stdin from our channel and forwards it to the remote process
+        let (abort_req_task_tx, mut abort_req_task_rx) = mpsc::channel::<()>(1);
         let req_task = tokio::spawn(async move {
-            process_outgoing_requests(tenant, id, session, stdin_rx, kill_rx).await
+            tokio::select! {
+                _ = abort_req_task_rx.recv() => {
+                    panic!("killed");
+                }
+                res = process_outgoing_requests(tenant, id, channel, stdin_rx, kill_rx) => {
+                    res
+                }
+            }
+        });
+
+        let status = Arc::new(RwLock::new(None));
+        let status_2 = Arc::clone(&status);
+        let wait_task = tokio::spawn(async move {
+            let res = match tokio::try_join!(req_task, res_task) {
+                Ok((_, res)) => res,
+                Err(x) => Err(RemoteProcessError::from(x)),
+            };
+            status_2.write().await.replace(res);
         });
 
         Ok(Self {
             id,
-            req_task,
-            res_task,
+            origin_id,
+            abort_req_task_tx,
+            abort_res_task_tx,
             stdin: Some(RemoteStdin(stdin_tx)),
             stdout: Some(RemoteStdout(stdout_rx)),
             stderr: Some(RemoteStderr(stderr_rx)),
             kill: kill_tx,
+            wait_task,
+            status,
         })
     }
 
@@ -125,20 +194,36 @@ impl RemoteProcess {
         self.id
     }
 
+    /// Checks if the process has completed, returning the exit status if it has, without
+    /// consuming the process itself. Note that this does not include join errors that can
+    /// occur when aborting and instead converts any error to a status of false. To acquire
+    /// the actual error, you must call `wait`
+    pub async fn status(&self) -> Option<(bool, Option<i32>)> {
+        self.status.read().await.as_ref().map(|x| match x {
+            Ok((success, exit_code)) => (*success, *exit_code),
+            Err(_) => (false, None),
+        })
+    }
+
     /// Waits for the process to terminate, returning the success status and an optional exit code
     pub async fn wait(self) -> Result<(bool, Option<i32>), RemoteProcessError> {
-        match tokio::try_join!(self.req_task, self.res_task) {
-            Ok((_, res)) => res,
-            Err(x) => Err(RemoteProcessError::from(x)),
-        }
+        // Wait for the process to complete before we try to get the status
+        let _ = self.wait_task.await;
+
+        // NOTE: If we haven't received an exit status, this lines up with the UnexpectedEof error
+        self.status
+            .write()
+            .await
+            .take()
+            .unwrap_or(Err(RemoteProcessError::UnexpectedEof))
     }
 
     /// Aborts the process by forcing its response task to shutdown, which means that a call
     /// to `wait` will return an error. Note that this does **not** send a kill request, so if
     /// you want to be nice you should send the request before aborting.
     pub fn abort(&self) {
-        self.req_task.abort();
-        self.res_task.abort();
+        let _ = self.abort_req_task_tx.try_send(());
+        let _ = self.abort_res_task_tx.try_send(());
     }
 
     /// Submits a kill request for the running process
@@ -156,6 +241,17 @@ impl RemoteProcess {
 pub struct RemoteStdin(mpsc::Sender<String>);
 
 impl RemoteStdin {
+    /// Tries to write to the stdin of the remote process, returning ok if immediately
+    /// successful, `WouldBlock` if would need to wait to send data, and `BrokenPipe`
+    /// if stdin has been closed
+    pub fn try_write(&mut self, data: impl Into<String>) -> io::Result<()> {
+        match self.0.try_send(data.into()) {
+            Ok(data) => Ok(data),
+            Err(TrySendError::Full(_)) => Err(io::Error::from(io::ErrorKind::WouldBlock)),
+            Err(TrySendError::Closed(_)) => Err(io::Error::from(io::ErrorKind::BrokenPipe)),
+        }
+    }
+
     /// Writes data to the stdin of a specific remote process
     pub async fn write(&mut self, data: impl Into<String>) -> io::Result<()> {
         self.0
@@ -163,6 +259,11 @@ impl RemoteStdin {
             .await
             .map_err(|x| io::Error::new(io::ErrorKind::BrokenPipe, x))
     }
+
+    /// Checks if stdin has been closed
+    pub fn is_closed(&self) -> bool {
+        self.0.is_closed()
+    }
 }
 
 /// A handle to a remote process' standard output (stdout)
@@ -170,7 +271,18 @@ impl RemoteStdin {
 pub struct RemoteStdout(mpsc::Receiver<String>);
 
 impl RemoteStdout {
-    /// Retrieves the latest stdout for a specific remote process
+    /// Tries to receive latest stdout for a remote process, yielding `None`
+    /// if no stdout is available, and `BrokenPipe` if stdout has been closed
+    pub fn try_read(&mut self) -> io::Result<Option<String>> {
+        match self.0.try_recv() {
+            Ok(data) => Ok(Some(data)),
+            Err(TryRecvError::Empty) => Ok(None),
+            Err(TryRecvError::Disconnected) => Err(io::Error::from(io::ErrorKind::BrokenPipe)),
+        }
+    }
+
+    /// Retrieves the latest stdout for a specific remote process, and `BrokenPipe` if stdout has
+    /// been closed
     pub async fn read(&mut self) -> io::Result<String> {
         self.0
             .recv()
@@ -184,7 +296,18 @@ impl RemoteStdout {
 pub struct RemoteStderr(mpsc::Receiver<String>);
 
 impl RemoteStderr {
-    /// Retrieves the latest stderr for a specific remote process
+    /// Tries to receive latest stderr for a remote process, yielding `None`
+    /// if no stderr is available, and `BrokenPipe` if stderr has been closed
+    pub fn try_read(&mut self) -> io::Result<Option<String>> {
+        match self.0.try_recv() {
+            Ok(data) => Ok(Some(data)),
+            Err(TryRecvError::Empty) => Ok(None),
+            Err(TryRecvError::Disconnected) => Err(io::Error::from(io::ErrorKind::BrokenPipe)),
+        }
+    }
+
+    /// Retrieves the latest stderr for a specific remote process, and `BrokenPipe` if stderr has
+    /// been closed
     pub async fn read(&mut self) -> io::Result<String> {
         self.0
             .recv()
@@ -195,21 +318,18 @@ impl RemoteStderr {
 
 /// Helper function that loops, processing outgoing stdin requests to a remote process as well as
 /// supporting a kill request to terminate the remote process
-async fn process_outgoing_requests<T>(
+async fn process_outgoing_requests(
     tenant: String,
     id: usize,
-    mut session: Session<T>,
+    mut channel: SessionChannel,
     mut stdin_rx: mpsc::Receiver<String>,
     mut kill_rx: mpsc::Receiver<()>,
-) -> Result<(), RemoteProcessError>
-where
-    T: DataStream,
-{
+) -> Result<(), RemoteProcessError> {
     let result = loop {
         tokio::select! {
             data = stdin_rx.recv() => {
                 match data {
-                    Some(data) => session.fire(
+                    Some(data) => channel.fire(
                         Request::new(
                             tenant.as_str(),
                             vec![RequestData::ProcStdin { id, data }]
@@ -220,12 +340,10 @@ where
             }
             msg = kill_rx.recv() => {
                 if msg.is_some() {
-                    session
-                        .fire(Request::new(
-                            tenant.as_str(),
-                            vec![RequestData::ProcKill { id }],
-                        ))
-                        .await?;
+                    channel.fire(Request::new(
+                        tenant.as_str(),
+                        vec![RequestData::ProcKill { id }],
+                    )).await?;
                     break Ok(());
                 } else {
                     break Err(RemoteProcessError::ChannelDead);
@@ -241,12 +359,12 @@ where
 /// Helper function that loops, processing incoming stdout & stderr requests from a remote process
 async fn process_incoming_responses(
     proc_id: usize,
-    mut broadcast: mpsc::Receiver<Response>,
+    mut mailbox: Mailbox,
     stdout_tx: mpsc::Sender<String>,
     stderr_tx: mpsc::Sender<String>,
     kill_tx: mpsc::Sender<()>,
 ) -> Result<(bool, Option<i32>), RemoteProcessError> {
-    while let Some(res) = broadcast.recv().await {
+    while let Some(res) = mailbox.next().await {
         // Check if any of the payload data is the termination
         let exit_status = res.payload.iter().find_map(|data| match data {
             ResponseData::ProcDone { id, success, code } if *id == proc_id => {
@@ -289,58 +407,72 @@ async fn process_incoming_responses(
 mod tests {
     use super::*;
     use crate::{
-        data::{Error, ErrorKind},
-        net::{InmemoryStream, Transport},
+        client::Session,
+        data::{Error, ErrorKind, Response},
+        net::{InmemoryStream, PlainCodec, Transport},
     };
+    use std::time::Duration;
 
-    fn make_session() -> (Transport<InmemoryStream>, Session<InmemoryStream>) {
+    fn make_session() -> (Transport<InmemoryStream, PlainCodec>, Session) {
         let (t1, t2) = Transport::make_pair();
         (t1, Session::initialize(t2).unwrap())
     }
 
     #[tokio::test]
-    async fn spawn_should_return_bad_response_if_payload_size_unexpected() {
+    async fn spawn_should_return_invalid_data_if_payload_size_unexpected() {
         let (mut transport, session) = make_session();
 
         // Create a task for process spawning as we need to handle the request and a response
         // in a separate async block
-        let spawn_task = tokio::spawn(RemoteProcess::spawn(
-            String::from("test-tenant"),
-            session,
-            String::from("cmd"),
-            vec![String::from("arg")],
-        ));
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
 
         // Wait until we get the request from the session
         let req = transport.receive::<Request>().await.unwrap().unwrap();
 
         // Send back a response through the session
         transport
-            .send(Response::new("test-tenant", Some(req.id), Vec::new()))
+            .send(Response::new("test-tenant", req.id, Vec::new()))
             .await
             .unwrap();
 
         // Get the spawn result and verify
         let result = spawn_task.await.unwrap();
         assert!(
-            matches!(result, Err(RemoteProcessError::BadResponse)),
+            matches!(
+                &result,
+                Err(RemoteProcessError::TransportError(TransportError::IoError(x)))
+                    if x.kind() == io::ErrorKind::InvalidData
+            ),
             "Unexpected result: {:?}",
             result
         );
     }
 
     #[tokio::test]
-    async fn spawn_should_return_bad_response_if_did_not_get_a_indicator_that_process_started() {
+    async fn spawn_should_return_invalid_data_if_did_not_get_a_indicator_that_process_started() {
         let (mut transport, session) = make_session();
 
         // Create a task for process spawning as we need to handle the request and a response
         // in a separate async block
-        let spawn_task = tokio::spawn(RemoteProcess::spawn(
-            String::from("test-tenant"),
-            session,
-            String::from("cmd"),
-            vec![String::from("arg")],
-        ));
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
 
         // Wait until we get the request from the session
         let req = transport.receive::<Request>().await.unwrap().unwrap();
@@ -349,9 +481,9 @@ mod tests {
         transport
             .send(Response::new(
                 "test-tenant",
-                Some(req.id),
+                req.id,
                 vec![ResponseData::Error(Error {
-                    kind: ErrorKind::Other,
+                    kind: ErrorKind::BrokenPipe,
                     description: String::from("some error"),
                 })],
             ))
@@ -361,7 +493,11 @@ mod tests {
         // Get the spawn result and verify
         let result = spawn_task.await.unwrap();
         assert!(
-            matches!(result, Err(RemoteProcessError::BadResponse)),
+            matches!(
+                &result,
+                Err(RemoteProcessError::TransportError(TransportError::IoError(x)))
+                    if x.kind() == io::ErrorKind::BrokenPipe
+            ),
             "Unexpected result: {:?}",
             result
         );
@@ -373,12 +509,16 @@ mod tests {
 
         // Create a task for process spawning as we need to handle the request and a response
         // in a separate async block
-        let spawn_task = tokio::spawn(RemoteProcess::spawn(
-            String::from("test-tenant"),
-            session,
-            String::from("cmd"),
-            vec![String::from("arg")],
-        ));
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
 
         // Wait until we get the request from the session
         let req = transport.receive::<Request>().await.unwrap().unwrap();
@@ -388,7 +528,7 @@ mod tests {
         transport
             .send(Response::new(
                 "test-tenant",
-                Some(req.id),
+                req.id,
                 vec![ResponseData::ProcStart { id }],
             ))
             .await
@@ -415,12 +555,16 @@ mod tests {
 
         // Create a task for process spawning as we need to handle the request and a response
         // in a separate async block
-        let spawn_task = tokio::spawn(RemoteProcess::spawn(
-            String::from("test-tenant"),
-            session,
-            String::from("cmd"),
-            vec![String::from("arg")],
-        ));
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
 
         // Wait until we get the request from the session
         let req = transport.receive::<Request>().await.unwrap().unwrap();
@@ -430,7 +574,7 @@ mod tests {
         transport
             .send(Response::new(
                 "test-tenant",
-                Some(req.id),
+                req.id,
                 vec![ResponseData::ProcStart { id }],
             ))
             .await
@@ -468,12 +612,16 @@ mod tests {
 
         // Create a task for process spawning as we need to handle the request and a response
         // in a separate async block
-        let spawn_task = tokio::spawn(RemoteProcess::spawn(
-            String::from("test-tenant"),
-            session,
-            String::from("cmd"),
-            vec![String::from("arg")],
-        ));
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
 
         // Wait until we get the request from the session
         let req = transport.receive::<Request>().await.unwrap().unwrap();
@@ -483,7 +631,7 @@ mod tests {
         transport
             .send(Response::new(
                 "test-tenant",
-                Some(req.id),
+                req.id,
                 vec![ResponseData::ProcStart { id }],
             ))
             .await
@@ -520,12 +668,16 @@ mod tests {
 
         // Create a task for process spawning as we need to handle the request and a response
         // in a separate async block
-        let spawn_task = tokio::spawn(RemoteProcess::spawn(
-            String::from("test-tenant"),
-            session,
-            String::from("cmd"),
-            vec![String::from("arg")],
-        ));
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
 
         // Wait until we get the request from the session
         let req = transport.receive::<Request>().await.unwrap().unwrap();
@@ -535,7 +687,7 @@ mod tests {
         transport
             .send(Response::new(
                 "test-tenant",
-                Some(req.id),
+                req.id,
                 vec![ResponseData::ProcStart { id }],
             ))
             .await
@@ -547,7 +699,7 @@ mod tests {
         transport
             .send(Response::new(
                 "test-tenant",
-                None,
+                req.id,
                 vec![ResponseData::ProcStdout {
                     id,
                     data: String::from("some out"),
@@ -566,12 +718,16 @@ mod tests {
 
         // Create a task for process spawning as we need to handle the request and a response
         // in a separate async block
-        let spawn_task = tokio::spawn(RemoteProcess::spawn(
-            String::from("test-tenant"),
-            session,
-            String::from("cmd"),
-            vec![String::from("arg")],
-        ));
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
 
         // Wait until we get the request from the session
         let req = transport.receive::<Request>().await.unwrap().unwrap();
@@ -581,7 +737,7 @@ mod tests {
         transport
             .send(Response::new(
                 "test-tenant",
-                Some(req.id),
+                req.id,
                 vec![ResponseData::ProcStart { id }],
             ))
             .await
@@ -593,7 +749,7 @@ mod tests {
         transport
             .send(Response::new(
                 "test-tenant",
-                None,
+                req.id,
                 vec![ResponseData::ProcStderr {
                     id,
                     data: String::from("some err"),
@@ -607,17 +763,21 @@ mod tests {
     }
 
     #[tokio::test]
-    async fn wait_should_return_error_if_internal_tasks_fail() {
+    async fn status_should_return_none_if_not_done() {
         let (mut transport, session) = make_session();
 
         // Create a task for process spawning as we need to handle the request and a response
         // in a separate async block
-        let spawn_task = tokio::spawn(RemoteProcess::spawn(
-            String::from("test-tenant"),
-            session,
-            String::from("cmd"),
-            vec![String::from("arg")],
-        ));
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
 
         // Wait until we get the request from the session
         let req = transport.receive::<Request>().await.unwrap().unwrap();
@@ -627,7 +787,146 @@ mod tests {
         transport
             .send(Response::new(
                 "test-tenant",
-                Some(req.id),
+                req.id,
+                vec![ResponseData::ProcStart { id }],
+            ))
+            .await
+            .unwrap();
+
+        // Receive the process and then check its status
+        let proc = spawn_task.await.unwrap().unwrap();
+
+        let result = proc.status().await;
+        assert_eq!(result, None, "Unexpectedly got proc status: {:?}", result);
+    }
+
+    #[tokio::test]
+    async fn status_should_return_false_for_success_if_internal_tasks_fail() {
+        let (mut transport, session) = make_session();
+
+        // Create a task for process spawning as we need to handle the request and a response
+        // in a separate async block
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
+
+        // Wait until we get the request from the session
+        let req = transport.receive::<Request>().await.unwrap().unwrap();
+
+        // Send back a response through the session
+        let id = 12345;
+        transport
+            .send(Response::new(
+                "test-tenant",
+                req.id,
+                vec![ResponseData::ProcStart { id }],
+            ))
+            .await
+            .unwrap();
+
+        // Receive the process and then abort it to make internal tasks fail
+        let proc = spawn_task.await.unwrap().unwrap();
+        proc.abort();
+
+        // Wait a bit to ensure the other tasks abort
+        tokio::time::sleep(Duration::from_millis(100)).await;
+
+        // Peek at the status to confirm the result
+        let result = proc.status().await;
+        match result {
+            Some((false, None)) => {}
+            x => panic!("Unexpected result: {:?}", x),
+        }
+    }
+
+    #[tokio::test]
+    async fn status_should_return_process_status_when_done() {
+        let (mut transport, session) = make_session();
+
+        // Create a task for process spawning as we need to handle the request and a response
+        // in a separate async block
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
+
+        // Wait until we get the request from the session
+        let req = transport.receive::<Request>().await.unwrap().unwrap();
+
+        // Send back a response through the session
+        let id = 12345;
+        transport
+            .send(Response::new(
+                "test-tenant",
+                req.id,
+                vec![ResponseData::ProcStart { id }],
+            ))
+            .await
+            .unwrap();
+
+        // Receive the process and then spawn a task for it to complete
+        let proc = spawn_task.await.unwrap().unwrap();
+
+        // Send a process completion response to pass along exit status and conclude wait
+        transport
+            .send(Response::new(
+                "test-tenant",
+                req.id,
+                vec![ResponseData::ProcDone {
+                    id,
+                    success: true,
+                    code: Some(123),
+                }],
+            ))
+            .await
+            .unwrap();
+
+        // Wait a bit to ensure the status gets transmitted
+        tokio::time::sleep(Duration::from_millis(100)).await;
+
+        // Finally, verify that we complete and get the expected results
+        assert_eq!(proc.status().await, Some((true, Some(123))));
+    }
+
+    #[tokio::test]
+    async fn wait_should_return_error_if_internal_tasks_fail() {
+        let (mut transport, session) = make_session();
+
+        // Create a task for process spawning as we need to handle the request and a response
+        // in a separate async block
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
+
+        // Wait until we get the request from the session
+        let req = transport.receive::<Request>().await.unwrap().unwrap();
+
+        // Send back a response through the session
+        let id = 12345;
+        transport
+            .send(Response::new(
+                "test-tenant",
+                req.id,
                 vec![ResponseData::ProcStart { id }],
             ))
             .await
@@ -651,12 +950,16 @@ mod tests {
 
         // Create a task for process spawning as we need to handle the request and a response
         // in a separate async block
-        let spawn_task = tokio::spawn(RemoteProcess::spawn(
-            String::from("test-tenant"),
-            session,
-            String::from("cmd"),
-            vec![String::from("arg")],
-        ));
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
 
         // Wait until we get the request from the session
         let req = transport.receive::<Request>().await.unwrap().unwrap();
@@ -666,7 +969,7 @@ mod tests {
         transport
             .send(Response::new(
                 "test-tenant",
-                Some(req.id),
+                req.id,
                 vec![ResponseData::ProcStart { id }],
             ))
             .await
@@ -674,6 +977,10 @@ mod tests {
 
         // Receive the process and then terminate session connection
         let proc = spawn_task.await.unwrap().unwrap();
+
+        // Ensure that the spawned task gets a chance to wait on stdout/stderr
+        tokio::task::yield_now().await;
+
         drop(transport);
 
         // Ensure that the other tasks are cancelled before continuing
@@ -693,12 +1000,16 @@ mod tests {
 
         // Create a task for process spawning as we need to handle the request and a response
         // in a separate async block
-        let spawn_task = tokio::spawn(RemoteProcess::spawn(
-            String::from("test-tenant"),
-            session,
-            String::from("cmd"),
-            vec![String::from("arg")],
-        ));
+        let spawn_task = tokio::spawn(async move {
+            RemoteProcess::spawn(
+                String::from("test-tenant"),
+                session.clone_channel(),
+                String::from("cmd"),
+                vec![String::from("arg")],
+                false,
+            )
+            .await
+        });
 
         // Wait until we get the request from the session
         let req = transport.receive::<Request>().await.unwrap().unwrap();
@@ -708,7 +1019,7 @@ mod tests {
         transport
             .send(Response::new(
                 "test-tenant",
-                Some(req.id),
+                req.id,
                 vec![ResponseData::ProcStart { id }],
             ))
             .await
@@ -722,7 +1033,7 @@ mod tests {
         transport
             .send(Response::new(
                 "test-tenant",
-                None,
+                req.id,
                 vec![ResponseData::ProcDone {
                     id,
                     success: false,
