@@ -222,6 +222,11 @@ pub enum RequestData {
 
         /// Arguments for the command
         args: Vec<String>,
+
+        /// Whether or not the process should be detached, meaning that the process will not be
+        /// killed when the associated client disconnects
+        #[cfg_attr(feature = "structopt", structopt(long))]
+        detached: bool,
     },
 
     /// Kills a process running on the remote machine
@@ -330,32 +335,7 @@ pub enum ResponseData {
     Exists(bool),
 
     /// Represents metadata about some filesystem object (file, directory, symlink) on remote machine
-    Metadata {
-        /// Canonicalized path to the file or directory, resolving symlinks, only included
-        /// if flagged during the request
-        canonicalized_path: Option<PathBuf>,
-
-        /// Represents the type of the entry as a file/dir/symlink
-        file_type: FileType,
-
-        /// Size of the file/directory/symlink in bytes
-        len: u64,
-
-        /// Whether or not the file/directory/symlink is marked as unwriteable
-        readonly: bool,
-
-        /// Represents the last time (in milliseconds) when the file/directory/symlink was accessed;
-        /// can be optional as certain systems don't support this
-        accessed: Option<u128>,
-
-        /// Represents when (in milliseconds) the file/directory/symlink was created;
-        /// can be optional as certain systems don't support this
-        created: Option<u128>,
-
-        /// Represents the last time (in milliseconds) when the file/directory/symlink was modified;
-        /// can be optional as certain systems don't support this
-        modified: Option<u128>,
-    },
+    Metadata(Metadata),
 
     /// Response to starting a new process
     ProcStart {
@@ -400,26 +380,59 @@ pub enum ResponseData {
     },
 
     /// Response to retrieving information about the server and the system it is on
-    SystemInfo {
-        /// Family of the operating system as described in
-        /// https://doc.rust-lang.org/std/env/consts/constant.FAMILY.html
-        family: String,
+    SystemInfo(SystemInfo),
+}
 
-        /// Name of the specific operating system as described in
-        /// https://doc.rust-lang.org/std/env/consts/constant.OS.html
-        os: String,
+/// Represents metadata about some path on a remote machine
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+pub struct Metadata {
+    /// Canonicalized path to the file or directory, resolving symlinks, only included
+    /// if flagged during the request
+    pub canonicalized_path: Option<PathBuf>,
 
-        /// Architecture of the CPI as described in
-        /// https://doc.rust-lang.org/std/env/consts/constant.ARCH.html
-        arch: String,
+    /// Represents the type of the entry as a file/dir/symlink
+    pub file_type: FileType,
 
-        /// Current working directory of the running server process
-        current_dir: PathBuf,
+    /// Size of the file/directory/symlink in bytes
+    pub len: u64,
 
-        /// Primary separator for path components for the current platform
-        /// as defined in https://doc.rust-lang.org/std/path/constant.MAIN_SEPARATOR.html
-        main_separator: char,
-    },
+    /// Whether or not the file/directory/symlink is marked as unwriteable
+    pub readonly: bool,
+
+    /// Represents the last time (in milliseconds) when the file/directory/symlink was accessed;
+    /// can be optional as certain systems don't support this
+    pub accessed: Option<u128>,
+
+    /// Represents when (in milliseconds) the file/directory/symlink was created;
+    /// can be optional as certain systems don't support this
+    pub created: Option<u128>,
+
+    /// Represents the last time (in milliseconds) when the file/directory/symlink was modified;
+    /// can be optional as certain systems don't support this
+    pub modified: Option<u128>,
+}
+
+/// Represents information about a system
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+pub struct SystemInfo {
+    /// Family of the operating system as described in
+    /// https://doc.rust-lang.org/std/env/consts/constant.FAMILY.html
+    pub family: String,
+
+    /// Name of the specific operating system as described in
+    /// https://doc.rust-lang.org/std/env/consts/constant.OS.html
+    pub os: String,
+
+    /// Architecture of the CPI as described in
+    /// https://doc.rust-lang.org/std/env/consts/constant.ARCH.html
+    pub arch: String,
+
+    /// Current working directory of the running server process
+    pub current_dir: PathBuf,
+
+    /// Primary separator for path components for the current platform
+    /// as defined in https://doc.rust-lang.org/std/path/constant.MAIN_SEPARATOR.html
+    pub main_separator: char,
 }
 
 /// Represents information about a single entry within a directory
@@ -457,6 +470,9 @@ pub struct RunningProcess {
     /// Arguments for the command
     pub args: Vec<String>,
 
+    /// Whether or not the process was run in detached mode
+    pub detached: bool,
+
     /// Arbitrary id associated with running process
     ///
     /// Not the same as the process' pid!
@@ -493,6 +509,8 @@ pub struct Error {
     pub description: String,
 }
 
+impl std::error::Error for Error {}
+
 impl From<io::Error> for Error {
     fn from(x: io::Error) -> Self {
         Self {
@@ -502,6 +520,12 @@ impl From<io::Error> for Error {
     }
 }
 
+impl From<Error> for io::Error {
+    fn from(x: Error) -> Self {
+        Self::new(x.kind.into(), x.description)
+    }
+}
+
 impl From<walkdir::Error> for Error {
     fn from(x: walkdir::Error) -> Self {
         if x.io_error().is_some() {
@@ -589,12 +613,6 @@ pub enum ErrorKind {
     /// reached prematurely
     UnexpectedEof,
 
-    /// This operation is unsupported on this platform
-    Unsupported,
-
-    /// An operation could not be completed, because it failed to allocate enough memory.
-    OutOfMemory,
-
     /// When a loop is encountered when walking a directory
     Loop,
 
@@ -629,8 +647,6 @@ impl From<io::ErrorKind> for ErrorKind {
             io::ErrorKind::Interrupted => Self::Interrupted,
             io::ErrorKind::Other => Self::Other,
             io::ErrorKind::UnexpectedEof => Self::UnexpectedEof,
-            io::ErrorKind::Unsupported => Self::Unsupported,
-            io::ErrorKind::OutOfMemory => Self::OutOfMemory,
 
             // This exists because io::ErrorKind is non_exhaustive
             _ => Self::Unknown,
@@ -638,6 +654,32 @@ impl From<io::ErrorKind> for ErrorKind {
     }
 }
 
+impl From<ErrorKind> for io::ErrorKind {
+    fn from(kind: ErrorKind) -> Self {
+        match kind {
+            ErrorKind::NotFound => Self::NotFound,
+            ErrorKind::PermissionDenied => Self::PermissionDenied,
+            ErrorKind::ConnectionRefused => Self::ConnectionRefused,
+            ErrorKind::ConnectionReset => Self::ConnectionReset,
+            ErrorKind::ConnectionAborted => Self::ConnectionAborted,
+            ErrorKind::NotConnected => Self::NotConnected,
+            ErrorKind::AddrInUse => Self::AddrInUse,
+            ErrorKind::AddrNotAvailable => Self::AddrNotAvailable,
+            ErrorKind::BrokenPipe => Self::BrokenPipe,
+            ErrorKind::AlreadyExists => Self::AlreadyExists,
+            ErrorKind::WouldBlock => Self::WouldBlock,
+            ErrorKind::InvalidInput => Self::InvalidInput,
+            ErrorKind::InvalidData => Self::InvalidData,
+            ErrorKind::TimedOut => Self::TimedOut,
+            ErrorKind::WriteZero => Self::WriteZero,
+            ErrorKind::Interrupted => Self::Interrupted,
+            ErrorKind::Other => Self::Other,
+            ErrorKind::UnexpectedEof => Self::UnexpectedEof,
+            _ => Self::Other,
+        }
+    }
+}
+
 /// Used to provide a default serde value of 1
 const fn one() -> usize {
     1
