@@ -1,7 +1,8 @@
 use crate::{
     constants::{MAX_PIPE_CHUNK_SIZE, READ_PAUSE_MILLIS},
     data::{
-        self, DirEntry, FileType, Request, RequestData, Response, ResponseData, RunningProcess,
+        self, DirEntry, FileType, Metadata, Request, RequestData, Response, ResponseData,
+        RunningProcess, SystemInfo,
     },
     server::distant::state::{Process, State},
 };
@@ -97,9 +98,13 @@ pub(super) async fn process(
                 canonicalize,
                 resolve_file_type,
             } => metadata(path, canonicalize, resolve_file_type).await,
-            RequestData::ProcRun { cmd, args } => proc_run(conn_id, state, reply, cmd, args).await,
-            RequestData::ProcKill { id } => proc_kill(state, id).await,
-            RequestData::ProcStdin { id, data } => proc_stdin(state, id, data).await,
+            RequestData::ProcRun {
+                cmd,
+                args,
+                detached,
+            } => proc_run(conn_id, state, reply, cmd, args, detached).await,
+            RequestData::ProcKill { id } => proc_kill(conn_id, state, id).await,
+            RequestData::ProcStdin { id, data } => proc_stdin(conn_id, state, id, data).await,
             RequestData::ProcList {} => proc_list(state).await,
             RequestData::SystemInfo {} => system_info().await,
         }
@@ -177,6 +182,7 @@ async fn file_write(path: PathBuf, data: impl AsRef<[u8]>) -> Result<Outgoing, S
 
 async fn file_append(path: PathBuf, data: impl AsRef<[u8]>) -> Result<Outgoing, ServerError> {
     let mut file = tokio::fs::OpenOptions::new()
+        .create(true)
         .append(true)
         .open(path)
         .await?;
@@ -388,7 +394,7 @@ async fn metadata(
         metadata.file_type()
     };
 
-    Ok(Outgoing::from(ResponseData::Metadata {
+    Ok(Outgoing::from(ResponseData::Metadata(Metadata {
         canonicalized_path,
         accessed: metadata
             .accessed()
@@ -414,7 +420,7 @@ async fn metadata(
         } else {
             FileType::Symlink
         },
-    }))
+    })))
 }
 
 async fn proc_run<F>(
@@ -423,6 +429,7 @@ async fn proc_run<F>(
     reply: F,
     cmd: String,
     args: Vec<String>,
+    detached: bool,
 ) -> Result<Outgoing, ServerError>
 where
     F: FnMut(Vec<ResponseData>) -> ReplyRet + Clone + Send + 'static,
@@ -438,7 +445,7 @@ where
     state
         .lock()
         .await
-        .push_process(conn_id, Process::new(id, cmd, args));
+        .push_process(conn_id, Process::new(id, cmd, args, detached));
 
     let post_hook = Box::new(move |mut state_lock: MutexGuard<'_, State>| {
         // Spawn a task that sends stdout as a response
@@ -452,7 +459,7 @@ where
                         Ok(data) => {
                             let payload = vec![ResponseData::ProcStdout { id, data }];
                             if !reply_2(payload).await {
-                                error!("<Conn @ {}> Stdout channel closed", conn_id);
+                                error!("<Conn @ {} | Proc {}> Stdout channel closed", conn_id, id);
                                 break;
                             }
 
@@ -464,12 +471,21 @@ where
                             .await;
                         }
                         Err(x) => {
-                            error!("Invalid data read from stdout pipe: {}", x);
+                            error!(
+                                "<Conn @ {} | Proc {}> Invalid data read from stdout pipe: {}",
+                                conn_id, id, x
+                            );
                             break;
                         }
                     },
                     Ok(_) => break,
-                    Err(_) => break,
+                    Err(x) => {
+                        error!(
+                            "<Conn @ {} | Proc {}> Reading stdout failed: {}",
+                            conn_id, id, x
+                        );
+                        break;
+                    }
                 }
             }
         });
@@ -485,7 +501,7 @@ where
                         Ok(data) => {
                             let payload = vec![ResponseData::ProcStderr { id, data }];
                             if !reply_2(payload).await {
-                                error!("<Conn @ {}> Stderr channel closed", conn_id);
+                                error!("<Conn @ {} | Proc {}> Stderr channel closed", conn_id, id);
                                 break;
                             }
 
@@ -497,12 +513,21 @@ where
                             .await;
                         }
                         Err(x) => {
-                            error!("Invalid data read from stdout pipe: {}", x);
+                            error!(
+                                "<Conn @ {} | Proc {}> Invalid data read from stdout pipe: {}",
+                                conn_id, id, x
+                            );
                             break;
                         }
                     },
                     Ok(_) => break,
-                    Err(_) => break,
+                    Err(x) => {
+                        error!(
+                            "<Conn @ {} | Proc {}> Reading stderr failed: {}",
+                            conn_id, id, x
+                        );
+                        break;
+                    }
                 }
             }
         });
@@ -514,7 +539,7 @@ where
             while let Some(line) = stdin_rx.recv().await {
                 if let Err(x) = stdin.write_all(line.as_bytes()).await {
                     error!(
-                        "<Conn @ {}> Failed to send stdin to process {}: {}",
+                        "<Conn @ {} | Proc {}> Failed to send stdin: {}",
                         conn_id, id, x
                     );
                     break;
@@ -530,18 +555,22 @@ where
         let wait_task = tokio::spawn(async move {
             tokio::select! {
                 status = child.wait() => {
-                    debug!("<Conn @ {}> Process {} done", conn_id, id);
+                    debug!(
+                        "<Conn @ {} | Proc {}> Completed and waiting on stdout & stderr tasks",
+                        conn_id,
+                        id,
+                    );
 
                     // Force stdin task to abort if it hasn't exited as there is no
                     // point to sending any more stdin
                     stdin_task.abort();
 
                     if let Err(x) = stderr_task.await {
-                        error!("<Conn @ {}> Join on stderr task failed: {}", conn_id, x);
+                        error!("<Conn @ {} | Proc {}> Join on stderr task failed: {}", conn_id, id, x);
                     }
 
                     if let Err(x) = stdout_task.await {
-                        error!("<Conn @ {}> Join on stdout task failed: {}", conn_id, x);
+                        error!("<Conn @ {} | Proc {}> Join on stdout task failed: {}", conn_id, id, x);
                     }
 
                     state_2.lock().await.remove_process(conn_id, id);
@@ -549,11 +578,18 @@ where
                     match status {
                         Ok(status) => {
                             let success = status.success();
-                            let code = status.code();
+                            let mut code = status.code();
+
+                            // If we succeeded and have no exit code, automatically populate
+                            // with success exit code
+                            if success && code.is_none() {
+                                code = Some(0);
+                            }
+
                             let payload = vec![ResponseData::ProcDone { id, success, code }];
                             if !reply_2(payload).await {
                                 error!(
-                                    "<Conn @ {}> Failed to send done for process {}!",
+                                    "<Conn @ {} | Proc {}> Failed to send done",
                                     conn_id,
                                     id,
                                 );
@@ -563,7 +599,7 @@ where
                             let payload = vec![ResponseData::from(x)];
                             if !reply_2(payload).await {
                                 error!(
-                                    "<Conn @ {}> Failed to send error for waiting on process {}!",
+                                    "<Conn @ {} | Proc {}> Failed to send error for waiting",
                                     conn_id,
                                     id,
                                 );
@@ -573,10 +609,10 @@ where
 
                 },
                 _ = kill_rx => {
-                    debug!("<Conn @ {}> Process {} killed", conn_id, id);
+                    debug!("<Conn @ {} | Proc {}> Killing", conn_id, id);
 
                     if let Err(x) = child.kill().await {
-                        error!("<Conn @ {}> Unable to kill process {}: {}", conn_id, id, x);
+                        error!("<Conn @ {} | Proc {}> Unable to kill: {}", conn_id, id, x);
                     }
 
                     // Force stdin task to abort if it hasn't exited as there is no
@@ -584,18 +620,24 @@ where
                     stdin_task.abort();
 
                     if let Err(x) = stderr_task.await {
-                        error!("<Conn @ {}> Join on stderr task failed: {}", conn_id, x);
+                        error!("<Conn @ {} | Proc {}> Join on stderr task failed: {}", conn_id, id, x);
                     }
 
                     if let Err(x) = stdout_task.await {
-                        error!("<Conn @ {}> Join on stdout task failed: {}", conn_id, x);
+                        error!("<Conn @ {} | Proc {}> Join on stdout task failed: {}", conn_id, id, x);
+                    }
+
+                    // Wait for the child after being killed to ensure that it has been cleaned
+                    // up at the operating system level
+                    if let Err(x) = child.wait().await {
+                        error!("<Conn @ {} | Proc {}> Failed to wait after killed: {}", conn_id, id, x);
                     }
 
                     state_2.lock().await.remove_process(conn_id, id);
 
                     let payload = vec![ResponseData::ProcDone { id, success: false, code: None }];
                     if !reply_2(payload).await {
-                        error!("<Conn @ {}> Failed to send done for process {}!", conn_id, id);
+                        error!("<Conn @ {} | Proc {}> Failed to send done", conn_id, id);
                     }
                 }
             }
@@ -613,7 +655,7 @@ where
     })
 }
 
-async fn proc_kill(state: HState, id: usize) -> Result<Outgoing, ServerError> {
+async fn proc_kill(conn_id: usize, state: HState, id: usize) -> Result<Outgoing, ServerError> {
     if let Some(process) = state.lock().await.processes.remove(&id) {
         if process.kill() {
             return Ok(Outgoing::from(ResponseData::Ok));
@@ -622,11 +664,19 @@ async fn proc_kill(state: HState, id: usize) -> Result<Outgoing, ServerError> {
 
     Err(ServerError::IoError(io::Error::new(
         io::ErrorKind::BrokenPipe,
-        "Unable to send kill signal to process",
+        format!(
+            "<Conn @ {} | Proc {}> Unable to send kill signal to process",
+            conn_id, id
+        ),
     )))
 }
 
-async fn proc_stdin(state: HState, id: usize, data: String) -> Result<Outgoing, ServerError> {
+async fn proc_stdin(
+    conn_id: usize,
+    state: HState,
+    id: usize,
+    data: String,
+) -> Result<Outgoing, ServerError> {
     if let Some(process) = state.lock().await.processes.get(&id) {
         if process.send_stdin(data).await {
             return Ok(Outgoing::from(ResponseData::Ok));
@@ -635,7 +685,10 @@ async fn proc_stdin(state: HState, id: usize, data: String) -> Result<Outgoing,
 
     Err(ServerError::IoError(io::Error::new(
         io::ErrorKind::BrokenPipe,
-        "Unable to send stdin to process",
+        format!(
+            "<Conn @ {} | Proc {}> Unable to send stdin to process",
+            conn_id, id,
+        ),
     )))
 }
 
@@ -649,6 +702,7 @@ async fn proc_list(state: HState) -> Result<Outgoing, ServerError> {
             .map(|p| RunningProcess {
                 cmd: p.cmd.to_string(),
                 args: p.args.clone(),
+                detached: p.detached,
                 id: p.id,
             })
             .collect(),
@@ -656,74 +710,81 @@ async fn proc_list(state: HState) -> Result<Outgoing, ServerError> {
 }
 
 async fn system_info() -> Result<Outgoing, ServerError> {
-    Ok(Outgoing::from(ResponseData::SystemInfo {
+    Ok(Outgoing::from(ResponseData::SystemInfo(SystemInfo {
         family: env::consts::FAMILY.to_string(),
         os: env::consts::OS.to_string(),
         arch: env::consts::ARCH.to_string(),
         current_dir: env::current_dir().unwrap_or_default(),
         main_separator: std::path::MAIN_SEPARATOR,
-    }))
+    })))
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
     use assert_fs::prelude::*;
+    use once_cell::sync::Lazy;
     use predicates::prelude::*;
     use std::time::Duration;
 
-    lazy_static::lazy_static! {
-        static ref TEMP_SCRIPT_DIR: assert_fs::TempDir = assert_fs::TempDir::new().unwrap();
-        static ref SCRIPT_RUNNER: String = String::from("bash");
+    static TEMP_SCRIPT_DIR: Lazy<assert_fs::TempDir> =
+        Lazy::new(|| assert_fs::TempDir::new().unwrap());
+    static SCRIPT_RUNNER: Lazy<String> = Lazy::new(|| String::from("bash"));
 
-        static ref ECHO_ARGS_TO_STDOUT_SH: assert_fs::fixture::ChildPath = {
-            let script = TEMP_SCRIPT_DIR.child("echo_args_to_stdout.sh");
-            script.write_str(indoc::indoc!(r#"
+    static ECHO_ARGS_TO_STDOUT_SH: Lazy<assert_fs::fixture::ChildPath> = Lazy::new(|| {
+        let script = TEMP_SCRIPT_DIR.child("echo_args_to_stdout.sh");
+        script
+            .write_str(indoc::indoc!(
+                r#"
                 #/usr/bin/env bash
-                printf "%s" "$@"
-            "#)).unwrap();
-            script
-        };
+                printf "%s" "$*"
+            "#
+            ))
+            .unwrap();
+        script
+    });
 
-        static ref ECHO_ARGS_TO_STDERR_SH: assert_fs::fixture::ChildPath = {
-            let script = TEMP_SCRIPT_DIR.child("echo_args_to_stderr.sh");
-            script.write_str(indoc::indoc!(r#"
+    static ECHO_ARGS_TO_STDERR_SH: Lazy<assert_fs::fixture::ChildPath> = Lazy::new(|| {
+        let script = TEMP_SCRIPT_DIR.child("echo_args_to_stderr.sh");
+        script
+            .write_str(indoc::indoc!(
+                r#"
                 #/usr/bin/env bash
-                printf "%s" "$@" 1>&2
-            "#)).unwrap();
-            script
-        };
+                printf "%s" "$*" 1>&2
+            "#
+            ))
+            .unwrap();
+        script
+    });
 
-        static ref ECHO_STDIN_TO_STDOUT_SH: assert_fs::fixture::ChildPath = {
-            let script = TEMP_SCRIPT_DIR.child("echo_stdin_to_stdout.sh");
-            script.write_str(indoc::indoc!(r#"
+    static ECHO_STDIN_TO_STDOUT_SH: Lazy<assert_fs::fixture::ChildPath> = Lazy::new(|| {
+        let script = TEMP_SCRIPT_DIR.child("echo_stdin_to_stdout.sh");
+        script
+            .write_str(indoc::indoc!(
+                r#"
                 #/usr/bin/env bash
                 while IFS= read; do echo "$REPLY"; done
-            "#)).unwrap();
-            script
-        };
+            "#
+            ))
+            .unwrap();
+        script
+    });
 
-        static ref EXIT_CODE_SH: assert_fs::fixture::ChildPath = {
-            let script = TEMP_SCRIPT_DIR.child("exit_code.sh");
-            script.write_str(indoc::indoc!(r#"
-                #!/usr/bin/env bash
-                exit "$1"
-            "#)).unwrap();
-            script
-        };
-
-        static ref SLEEP_SH: assert_fs::fixture::ChildPath = {
-            let script = TEMP_SCRIPT_DIR.child("sleep.sh");
-            script.write_str(indoc::indoc!(r#"
+    static SLEEP_SH: Lazy<assert_fs::fixture::ChildPath> = Lazy::new(|| {
+        let script = TEMP_SCRIPT_DIR.child("sleep.sh");
+        script
+            .write_str(indoc::indoc!(
+                r#"
                 #!/usr/bin/env bash
                 sleep "$1"
-            "#)).unwrap();
-            script
-        };
+            "#
+            ))
+            .unwrap();
+        script
+    });
 
-        static ref DOES_NOT_EXIST_BIN: assert_fs::fixture::ChildPath =
-            TEMP_SCRIPT_DIR.child("does_not_exist_bin");
-    }
+    static DOES_NOT_EXIST_BIN: Lazy<assert_fs::fixture::ChildPath> =
+        Lazy::new(|| TEMP_SCRIPT_DIR.child("does_not_exist_bin"));
 
     fn setup(
         buffer: usize,
@@ -989,6 +1050,40 @@ mod tests {
         file.assert(predicate::path::missing());
     }
 
+    #[tokio::test]
+    async fn file_append_should_create_file_if_missing() {
+        let (conn_id, state, tx, mut rx) = setup(1);
+
+        // Don't create the file directly, but define path
+        // where the file should be
+        let temp = assert_fs::TempDir::new().unwrap();
+        let file = temp.child("test-file");
+
+        let req = Request::new(
+            "test-tenant",
+            vec![RequestData::FileAppend {
+                path: file.path().to_path_buf(),
+                data: b"some extra contents".to_vec(),
+            }],
+        );
+
+        process(conn_id, state, req, tx).await.unwrap();
+
+        let res = rx.recv().await.unwrap();
+        assert_eq!(res.payload.len(), 1, "Wrong payload size");
+        assert!(
+            matches!(res.payload[0], ResponseData::Ok),
+            "Unexpected response: {:?}",
+            res.payload[0]
+        );
+
+        // Yield to allow chance to finish appending to file
+        tokio::time::sleep(Duration::from_millis(50)).await;
+
+        // Also verify that we actually did create to the file
+        file.assert("some extra contents");
+    }
+
     #[tokio::test]
     async fn file_append_should_send_ok_when_successful() {
         let (conn_id, state, tx, mut rx) = setup(1);
@@ -1054,6 +1149,40 @@ mod tests {
         file.assert(predicate::path::missing());
     }
 
+    #[tokio::test]
+    async fn file_append_text_should_create_file_if_missing() {
+        let (conn_id, state, tx, mut rx) = setup(1);
+
+        // Don't create the file directly, but define path
+        // where the file should be
+        let temp = assert_fs::TempDir::new().unwrap();
+        let file = temp.child("test-file");
+
+        let req = Request::new(
+            "test-tenant",
+            vec![RequestData::FileAppendText {
+                path: file.path().to_path_buf(),
+                text: "some extra contents".to_string(),
+            }],
+        );
+
+        process(conn_id, state, req, tx).await.unwrap();
+
+        let res = rx.recv().await.unwrap();
+        assert_eq!(res.payload.len(), 1, "Wrong payload size");
+        assert!(
+            matches!(res.payload[0], ResponseData::Ok),
+            "Unexpected response: {:?}",
+            res.payload[0]
+        );
+
+        // Yield to allow chance to finish appending to file
+        tokio::time::sleep(Duration::from_millis(50)).await;
+
+        // Also verify that we actually did create to the file
+        file.assert("some extra contents");
+    }
+
     #[tokio::test]
     async fn file_append_text_should_send_ok_when_successful() {
         let (conn_id, state, tx, mut rx) = setup(1);
@@ -1917,13 +2046,13 @@ mod tests {
         assert!(
             matches!(
                 res.payload[0],
-                ResponseData::Metadata {
+                ResponseData::Metadata(Metadata {
                     canonicalized_path: None,
                     file_type: FileType::File,
                     len: 9,
                     readonly: false,
                     ..
-                }
+                })
             ),
             "Unexpected response: {:?}",
             res.payload[0]
@@ -1953,12 +2082,12 @@ mod tests {
         assert!(
             matches!(
                 res.payload[0],
-                ResponseData::Metadata {
+                ResponseData::Metadata(Metadata {
                     canonicalized_path: None,
                     file_type: FileType::Dir,
                     readonly: false,
                     ..
-                }
+                })
             ),
             "Unexpected response: {:?}",
             res.payload[0]
@@ -1991,12 +2120,12 @@ mod tests {
         assert!(
             matches!(
                 res.payload[0],
-                ResponseData::Metadata {
+                ResponseData::Metadata(Metadata {
                     canonicalized_path: None,
                     file_type: FileType::Symlink,
                     readonly: false,
                     ..
-                }
+                })
             ),
             "Unexpected response: {:?}",
             res.payload[0]
@@ -2027,12 +2156,12 @@ mod tests {
         let res = rx.recv().await.unwrap();
         assert_eq!(res.payload.len(), 1, "Wrong payload size");
         match &res.payload[0] {
-            ResponseData::Metadata {
+            ResponseData::Metadata(Metadata {
                 canonicalized_path: Some(path),
                 file_type: FileType::Symlink,
                 readonly: false,
                 ..
-            } => assert_eq!(
+            }) => assert_eq!(
                 path,
                 &file.path().canonicalize().unwrap(),
                 "Symlink canonicalized path does not match referenced file"
@@ -2065,10 +2194,10 @@ mod tests {
         let res = rx.recv().await.unwrap();
         assert_eq!(res.payload.len(), 1, "Wrong payload size");
         match &res.payload[0] {
-            ResponseData::Metadata {
+            ResponseData::Metadata(Metadata {
                 file_type: FileType::File,
                 ..
-            } => {}
+            }) => {}
             x => panic!("Unexpected response: {:?}", x),
         }
     }
@@ -2082,6 +2211,7 @@ mod tests {
             vec![RequestData::ProcRun {
                 cmd: DOES_NOT_EXIST_BIN.to_str().unwrap().to_string(),
                 args: Vec::new(),
+                detached: false,
             }],
         );
 
@@ -2107,6 +2237,7 @@ mod tests {
             vec![RequestData::ProcRun {
                 cmd: SCRIPT_RUNNER.to_string(),
                 args: vec![ECHO_ARGS_TO_STDOUT_SH.to_str().unwrap().to_string()],
+                detached: false,
             }],
         );
 
@@ -2139,6 +2270,7 @@ mod tests {
                     ECHO_ARGS_TO_STDOUT_SH.to_str().unwrap().to_string(),
                     String::from("some stdout"),
                 ],
+                detached: false,
             }],
         );
 
@@ -2204,6 +2336,7 @@ mod tests {
                     ECHO_ARGS_TO_STDERR_SH.to_str().unwrap().to_string(),
                     String::from("some stderr"),
                 ],
+                detached: false,
             }],
         );
 
@@ -2266,6 +2399,7 @@ mod tests {
             vec![RequestData::ProcRun {
                 cmd: SCRIPT_RUNNER.to_string(),
                 args: vec![SLEEP_SH.to_str().unwrap().to_string(), String::from("0.1")],
+                detached: false,
             }],
         );
 
@@ -2308,6 +2442,7 @@ mod tests {
             vec![RequestData::ProcRun {
                 cmd: SCRIPT_RUNNER.to_string(),
                 args: vec![SLEEP_SH.to_str().unwrap().to_string(), String::from("1")],
+                detached: false,
             }],
         );
 
@@ -2382,6 +2517,7 @@ mod tests {
             vec![RequestData::ProcRun {
                 cmd: SCRIPT_RUNNER.to_string(),
                 args: vec![SLEEP_SH.to_str().unwrap().to_string(), String::from("1")],
+                detached: false,
             }],
         );
 
@@ -2477,6 +2613,7 @@ mod tests {
             vec![RequestData::ProcRun {
                 cmd: SCRIPT_RUNNER.to_string(),
                 args: vec![ECHO_STDIN_TO_STDOUT_SH.to_str().unwrap().to_string()],
+                detached: false,
             }],
         );
 
@@ -2549,6 +2686,7 @@ mod tests {
                 RequestData::ProcRun {
                     cmd: SCRIPT_RUNNER.to_string(),
                     args: vec![SLEEP_SH.to_str().unwrap().to_string(), String::from("1")],
+                    detached: false,
                 },
                 RequestData::ProcList {},
             ],
@@ -2572,6 +2710,7 @@ mod tests {
                 entries: vec![RunningProcess {
                     cmd: SCRIPT_RUNNER.to_string(),
                     args: vec![SLEEP_SH.to_str().unwrap().to_string(), String::from("1")],
+                    detached: false,
                     id,
                 }],
             },
@@ -2592,13 +2731,13 @@ mod tests {
         assert_eq!(res.payload.len(), 1, "Wrong payload size");
         assert_eq!(
             res.payload[0],
-            ResponseData::SystemInfo {
+            ResponseData::SystemInfo(SystemInfo {
                 family: env::consts::FAMILY.to_string(),
                 os: env::consts::OS.to_string(),
                 arch: env::consts::ARCH.to_string(),
                 current_dir: env::current_dir().unwrap_or_default(),
                 main_separator: std::path::MAIN_SEPARATOR,
-            },
+            }),
             "Unexpected response: {:?}",
             res.payload[0]
         );
